<?php
// $Id: viewers.admin.inc,v 1.1.2.6 2010/05/04 15:43:27 apanag Exp $

/**
 * @file
 * viewers.admin.inc
 */

function viewers_admin_settings() {

  $form['viewers_time'] = array(
    '#title' => t('Maximum time a user appears on the viewers list'),
    '#description' => t('Time is in seconds. Default is 60 secs.'),
    '#size' => '4',
    '#type' => 'textfield',
    '#default_value' => variable_get('viewers_time', '60'),
  );

  $form['viewers_pager'] = array(
    '#title' => t('Do you want to support pager'),
    '#description' => t('For example, do you want a page (node/100?page=1) to be different from its next page (node/100?page=2).If yes the module will display online users per page not per topic. Default is Yes.'),
    '#type' => 'radios',
    '#default_value' => variable_get('viewers_pager', 'yes'),
    '#options' => array(
      'yes' => t('Yes'),
      'no' => t('No'),
    ),
  );

  $form['viewers_users_roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Roles'),
    '#description' => t('Select the roles, which you want to enable tracking for.'),
    '#options' => user_roles(),
    '#default_value' => variable_get('viewers_users_roles', array('')),
  );

  $form['viewers_count_hide'] = array(
    '#title' => t('Count the hidden users, as "anonymous"'),
    '#type' => 'radios',
    '#description' => t('By default we count the hidden users, as anonymous users. If you select "No", the users are excluded from the total count.'),
    '#options' => array('0' => 'Yes', '1' => 'No'),
    '#default_value' => variable_get('viewers_count_hide', '0'),
  );

  $form['viewers_ajax'] = array(
    '#type' => 'fieldset',
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
    '#title' => 'Ajax Refresh',
  );

  $form['viewers_ajax']['viewers_ajax_refresh'] = array(
    '#title' => t('Do you want to support refresh via ajax'),
    '#type' => 'radios',
    '#description' => t('By default is disabled.'),
    '#default_value' => variable_get('viewers_ajax_refresh', 'no'),
    '#options' => array(
      'yes' => t('Yes'),
      'no' => t('No'),
    ),
  );

  $form['viewers_ajax']['viewers_ajax_refresh_time'] = array(
    '#title' => t('How often do you want to refresh the block'),
    '#type' => 'textfield',
    '#default_value' => variable_get('viewers_ajax_refresh_time', 60),
    '#size' => 4,
    '#description' => t('Time is in seconds. Default is 60 seconds.'),
  );

  $form['viewers_users_exclude_fieldset'] = array(
    '#type' => 'fieldset',
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
    '#title' => 'Users to exclude',
  );

  $form['viewers_users_exclude_fieldset']['viewers_users_exclude'] = array(
    '#title' => t('Which users to exclude'),
    '#type' => 'textarea',
    '#description' => t('Seperate uids with a comma, for example 1,2,34 etc. If by mistake you wrote the same uid twice, the validation saves only the first one. <br/><b>Attention:</b> Don\'t leave a spacebar char at the end of the uids.'),
    '#default_value' => variable_get('viewers_users_exclude', ''),
  );

  // Page specific visibility configurations.
  $access = user_access('use PHP for tracking visibility');
  $visibility = variable_get('viewers_visibility', 0);
  $pages = variable_get('viewers_pages', '');

  if (!$access) {
    $form['visibility'] = array('#type' => 'value', '#value' => 2);
    $form['pages'] = array('#type' => 'value', '#value' => $pages);
  }
  else {
    $options = array(t('Add to every page except the listed pages.'), t('Add to the listed pages only.'));
    $description = t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>'));
    }

  if ($access) {
    $options[] = t('Add if the following PHP code returns <code>TRUE</code> (PHP-mode, experts only).');
    $description .= ' '. t('If the PHP-mode is chosen, enter PHP code between %php. Note that executing incorrect PHP-code can break your Drupal site.', array('%php' => '<?php ?>'));

    $form['viewers_visibility_fieldset']= array(
      '#type' => 'fieldset',
      '#collapsed' => TRUE,
      '#collapsible' => TRUE,
      '#title' => 'Visibility Options',
    );

    $form['viewers_visibility_fieldset']['viewers_visibility'] = array(
      '#type' => 'radios',
      '#title' => t('Add tracking to specific pages'),
      '#options' => $options,
      '#default_value' => $visibility,
    );

    $form['viewers_visibility_fieldset']['viewers_pages'] = array(
      '#type' => 'textarea',
      '#title' => t('Pages'),
      '#default_value' => $pages,
      '#description' => $description,
      '#wysiwyg' => FALSE,
    );
  }

  foreach (node_get_types('names') as $name => $value) {

    $form['viewers_content_types_display']['viewers_content_type_'. $name] = array(
      '#title' => $value,
      '#type' => 'radios',
      '#default_value' => variable_get('viewers_content_type_'. $name, 2),
      '#options' => array(0 => 'Authenticated', 1 => 'Anonymous', 2 => 'Both'),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings')
  );

  return $form;
}

/**
 * Advanced settings for theming purposes
 */
function viewers_advanced_settings() {

  $status = variable_get('viewers_users_roles', '');
  //if the user has enabled and install colorpicker module add support for it:
  if (module_exists('colorpicker')) {
    foreach (user_roles() as $user => $name) {
      if ($status[$user] != 0 && $user != 1) {
        $form['viewers_user_color'][$user] = array(
          '#title' => $name,
          '#type' => (function_exists('colorpicker_2_or_later') ? 'colorpicker_' : '') .'textfield',
          '#description' => t('Only the selected roles are available. Default is #000000. Please enter a color value in the form #RRGGBB'),
          '#default_value' => variable_get('viewers_user_color_'. $user, '#000000'),
        );
      }
    }
  }

  $form['viewers_separator'] = array(
    '#title' => t('Choose separator to separate users'),
    '#type' => 'radios',
    '#default_value' => variable_get('viewers_separator', ', '),
    '#options' => array(
      ', ' => 'comma ( , )',
      ' - ' => 'hyphen ( - )',
      ' / ' => 'slash ( / )',
      ' ' => 'space (  )',
    ),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings')
  );

  return $form;
}

function viewers_advanced_settings_submit($form, &$form_state) {

  //set the variables
  variable_set('viewers_separator', $form_state['values']['viewers_separator']);
  foreach (user_roles() as $user => $name) {
    if ($form_state['values'][$name] != '#000000') {
      variable_set('viewers_user_color_'. $user, $form_state['values'][$user]);
    }
  }
}

function viewers_admin_settings_validate($form, &$form_state) {

  //Validate viewers_users_exclude
  $max_uid = db_result(db_query_range('SELECT uid FROM {users} ORDER BY uid DESC', 0, 1));
  $users = explode(',', $form_state['values']['viewers_users_exclude']);
  $unique = array_unique($users);
  if (!empty($users) && $users[0] != '') {
    foreach ($users as $uid) {
      if (!ctype_digit($uid) || ($uid < 0) || ($uid > $max_uid)) {
        form_set_error('viewers_users_exclude', t('Only valid uids are allowed'));
      }
    }
  }
  //if uids appeared multiple times, save unique uids only once
  $form_state['values']['viewers_users_exclude'] = implode(',', $unique);

  //validate viewers_time
  if (!ctype_digit($form_state['values']['viewers_time'])) {
    form_set_error('viewers_time', t('Only numbers are allowed.'));
  }
  elseif ($form_state['values']['viewers_time'] <= 0) {
    form_set_error('viewers_time', t('Time must be greater than zero.'));
  }
  if (!ctype_digit($form_state['values']['viewers_ajax_refresh_time'])) {
    form_set_error('viewers_ajax_refresh_time', t('Only numbers are allowed.'));
  }
  elseif ($form_state['values']['viewers_ajax_refresh_time'] <= 0) {
    form_set_error('viewers_ajax_refresh_time', t('Time must be greater than zero.'));
  }

  // Trim some text area values.
  $form_state['values']['viewers_pages'] = trim($form_state['values']['viewers_pages']);
}

function viewers_admin_settings_submit($form, &$form_state) {

  //if the configuration has changed delete the non-selected roles
  $sync = viewers_admin_settings_refresh_data($form_state['values']['viewers_users_roles']);
  if ($sync) {
    drupal_set_message(t('Tables have been synchronized.'));
  }

  //delete the excluded users from the database
  $message = viewers_admin_settings_exclude_users($form_state['values']['viewers_users_exclude']);
  if ($message != 0) {
    drupal_set_message(t($message .' users excluded.'));
    variable_set('viewers_users_exclude', $form_state['values']['viewers_users_exclude']);
  }
  else {
    variable_set('viewers_users_exclude', '');
  }

  //Save settings per content type
  foreach (node_get_types('names') as $name => $value) {
    variable_set('viewers_content_type_'. $name, $form_state['values']['viewers_content_type_'. $name]);
  }

  //Set the variables
  variable_set('viewers_time', $form_state['values']['viewers_time']);
  variable_set('viewers_pager', $form_state['values']['viewers_pager']);
  variable_set('viewers_ajax_refresh', $form_state['values']['viewers_ajax_refresh']);
  variable_set('viewers_users_roles', $form_state['values']['viewers_users_roles']);
  variable_set('viewers_users_exclude', $form_state['values']['viewers_users_exclude']);
  variable_set('viewers_pages', $form_state['values']['viewers_pages']);
  variable_set('viewers_visibility', $form_state['values']['viewers_visibility']);
  variable_set('viewers_count_hide', $form_state['values']['viewers_count_hide']);
  variable_set('viewers_users_roles_weights', $weights);
  variable_set('viewers_ajax_refresh_time', $form_state['values']['viewers_ajax_refresh_time']);

  drupal_set_message(t('Your settings have been saved.'));
}

/**
 * Delete the excluded users from the database
 */
function viewers_admin_settings_exclude_users($users) {
  $count = 0;
  $users = explode(',', $users);
  foreach ($users as $user => $uid) {
    //only count assigned uids
    if ($uid) {
      db_query('DELETE FROM {viewers} WHERE uid = %d', $uid);
      $count++;
    }
  }
  return $count;
}

/**
 * If the users changed its preferences delete the non-checked roles
 * If not return false
 */
function viewers_admin_settings_refresh_data($new_settings) {
  $users = variable_get('viewers_users_roles', '');
  $flag = FALSE;
  foreach ($new_settings as $role => $status) {
    if ($users[$role] != $status && $status == 0) {
      if ($role == 1) {//Delete anonymous users
        db_query('DELETE FROM {viewers} WHERE uid = %d', 0);
      }
      elseif ($role == 2) {//Delete all auth users
        db_query('DELETE FROM {viewers} WHERE uid <> %d', 0);
      }
      else {//Delete according to role id
        db_query('DELETE viewers.* FROM {viewers} INNER JOIN {users_roles} ON users_roles.uid = viewers.uid WHERE rid = %d', $role);
      }
      $flag = TRUE;
    }
  }
  return $flag;
}

function theme_viewers_admin_settings($form) {
  //theme viewers_content_types_display as radio buttons
  foreach (element_children($form['viewers_content_types_display']) as $key) {

    $row = array();

    $row['data'][0] = $form['viewers_content_types_display'][$key]['#title'];
    $row['data'][1] = drupal_render($form['viewers_content_types_display'][$key][0]);
    $row['data'][2] = drupal_render($form['viewers_content_types_display'][$key][1]);
    $row['data'][3] = drupal_render($form['viewers_content_types_display'][$key][2]);
    $rows[] = $row;

    drupal_render($form['viewers_content_types_display'][$key]);
  }

  $form['viewers_content_types_display'] = array(
    '#type' => 'fieldset',
    '#title' => 'Settings per Content Type',
    '#value' => theme('table', $header, $rows),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Choose which parts to display per content type. Default is both.'),
  );

  $output .= drupal_render($form);

  drupal_add_tabledrag('viewers-users-roles', 'order', 'sibling', 'viewers-users-roles-weight');

  return $output;
}